% This is the code for a synthetic experiment
% for the paper ...
% Note that matchpairs function used below might not
% be available in versions of matlab before 2019

n = 100;
m = 120;
d = 10;

% defining grid values for kappa_{in-in} and kappa_{in-out}
K1 = 30;
K2 = 50;
k_inin_min = .02;
k_inout_min = .1;
k_inin =  (1 + 0.1*(1:K1))*k_inin_min;
k_inout = (1 + 2*(1:K2))*k_inout_min;

% defining noise variances
sigma_max = 1;
sigma = sigma_max./(1:m).^(3/2);

tic
errLSL_cell = cell(1,3);
errLSS_cell = cell(1,3);
d = 10;

for n_d = 1:3
    n_rep = 1000;
    errLSS = zeros(K1,K2);
    errLSL = zeros(K1,K2);
    for i_rep = 1:n_rep
        for i=1:K1
            for j=1:K2
                %defining the noiseless feature vectors
                tdiese = zeros(d,m);
                tdiese(1,1:n) = (1:n)*k_inin(i);
                tdiese(1,(n+1):m) = n*k_inin(i) + (1:(m-n))*k_inout(j);

                % generating random feature-vectors
                xidiese = randn(d,m);
                xi = randn(d,n);
                Xdiese = tdiese + xidiese.*repmat(sigma,d,1);
                X = tdiese(:,1:n) + xi.*repmat(sigma(1:n),d,1);

                % computinf the distance matrix
                dist1 = repmat(sum(Xdiese.^2),n,1);    % n x m  matrix
                dist2 = repmat(sum(X.^2),m,1)';        % n x m  matrix
                cross = X'*Xdiese;                     % n x m matrix
                dist = dist1 - 2*cross + dist2;

                % LSS estimator
                [M,~,~] = matchpairs(dist,1e6);

                errLSS(i,j) = errLSS(i,j) + (sum(abs(M(:,2)-(1:n)'))>0);

                % LSL estimator
                [M,~,~] = matchpairs(log(dist),1e6);
                errLSL(i,j) = errLSL(i,j) + (sum(abs(M(:,2)-(1:n)'))>0);             
            end
        end
        if rem(i_rep,50)==0
            disp(['  ***  ', num2str(i_rep), ' replications done']);
            disp(['  ***   ', num2str(toc), 'sec']);
        end
    end
    errLSL_cell{n_d} = errLSL/n_rep;
    errLSS_cell{n_d} = errLSS/n_rep;
    d = d + d;
end
toc 
save errLSLcell errLSL_cell;
save errLSScell errLSS_cell;

load errLSLcell;
load errLSScell;

subplot(2,3,1)
image(errLSL_cell{1},'CDataMapping','scaled');
colormap copper;
caxis([0 1])
axis xy off ;

subplot(2,3,2)
image(errLSL_cell{2},'CDataMapping','scaled');
colormap copper;
caxis([0 1])
axis xy off ;


subplot(2,3,3)
image(errLSL_cell{3},'CDataMapping','scaled');
colormap copper;
caxis([0 1])
axis xy off ;


subplot(2,3,4)
image(errLSS_cell{1},'CDataMapping','scaled');
colormap copper;
caxis([0 1])
axis xy off ;

subplot(2,3,5)
image(errLSS_cell{2},'CDataMapping','scaled');
colormap copper;
caxis([0 1])
axis xy off ;

subplot(2,3,6)
image(errLSS_cell{3},'CDataMapping','scaled');
colormap copper;
caxis([0 1])
axis xy off ;
