# Optimal detection of the feature matching map in presence of noise and outliers 

This folder contains the code for running the real-data experiment (Experiment 3). The datasets were downloaded from https://www.cs.ubc.ca/~kmyi/imw2020/data.html. We've chosen 3 scenes (Brandenburg Gate, Temple Nara Japan and Reichstag) for this experiment. 

# Running matching algorithms 

### Brandenburg Gate

```sh
for o in  0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7
do
python baseline.py --dataset brandenburg_gate --skip_pairs 380 --num_query_kps 100 \
           --outlier_rate $o --log_file logs/brandenburg_gate_pairs_100kps_$o.log
done
```

### Temple Nara Japan 
```sh
for o in  0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7
do
python baseline.py --dataset temple_nara_japan --skip_pairs 180 --num_query_kps 100 \
           --outlier_rate $o --log_file logs/temple_nara_japan_pairs_100kps_$o.log
done
```

### Reichstag 
```sh
for o in  0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7
do
    python baseline.py --dataset reichstag --skip_pairs 1 --skip_kps 1 --num_query_kps 100 \
           --outlier_rate $o --log_file logs/reichstag_pairs_100kps_$o.log
done 
```

# Computing descriptor distances for boxplots 

### Brandenburg Gate
```sh
python baseline-descriptor-distance.py --dataset brandenburg_gate --skip_pairs 380 --num_query_kps 100 \
      --outlier_rate 0.0 --log_file logs/brandenburg_gate_dists.log
```

### Temple Nara Japan 
```sh
python baseline-descriptor-distance.py --dataset temple_nara_japan --skip_pairs 380 --num_query_kps 100 \
      --outlier_rate 0.0 --log_file logs/temple_nara_japan_dists.log
```

### Reichstag 
```sh
python baseline-descriptor-distance.py --dataset reichstag --skip_pairs 380 --num_query_kps 100 \
      --outlier_rate 0.0 --log_file logs/reichstag_dists.log
```

# Plotting 

The code for plotting accuracies measured in Hamming loss and boxplots for descriptor distances is provided in `exp3-illustrations.ipynb` Jupyter Notebook. 
